@@ -553,7 +553,7 @@ public:
             return;
         }
 #else
-        FCITX_UNUSED(name);
+        FCITX_UNUSED(fd);
 #endif
         throw dbus::MethodCallError("org.freedesktop.DBus.Error.InvalidArgs",
                                     "Wayland addon is not available.");
@@ -582,6 +582,9 @@ public:
             if (ic->focusGroup()) {
                 return true;
             }
+            if (std::string_view(ic->frontend()) == "dummy") {
+                return true;
+            }
             ss << "  IC [";
             for (auto v : ic->uuid()) {
                 ss << fmt::format("{:02x}", static_cast<int>(v));
