@@ -93,7 +93,8 @@ void DBusMenu::handleEvent(int32_t id) {
             return;
         }
 
-        parent_->instance()->setCurrentInputMethod(entry->uniqueName());
+        parent_->instance()->setCurrentInputMethod(
+            lastRelevantIc(), entry->uniqueName(), /*local=*/false);
     } else if (id >= BII_InputMethodGroupStart &&
                id <= BII_InputMethodGroupEnd) {
         size_t idx = id - BII_InputMethodGroupStart;
@@ -223,7 +224,7 @@ void DBusMenu::fillLayoutProperties(
     if (id < 0) {
         return;
     }
-    /* index == 0 means it has a sub menu */
+    /* id == 0 means it has a sub menu */
     auto &imManager = parent_->instance()->inputMethodManager();
     if (id == 0) {
         appendProperty(properties, propertyNames, "children-display",
